import React, { useEffect, useRef, useState } from 'react';
import { useImmer } from 'use-immer';
import { GraphQueryResult } from '../../../data-access/store';
import { LinkType, NodeType } from './types';
import { MatrixPixi } from './components/MatrixPixi';
import { VisualizationPropTypes, VISComponentType, VisualizationSettingsPropTypes } from '../../common';
import { Input } from '@graphpolaris/shared/lib/components/inputs';
import { GraphMetadata } from '@graphpolaris/shared/lib/data-access/statistics';
import { SettingsContainer } from '@graphpolaris/shared/lib/vis/components/config';

export interface MatrixVisProps {
  marks: string;
  color: string;
}

const settings: MatrixVisProps = {
  marks: 'rect',
  color: 'blue',
};

export const MatrixVis = React.memo(({ data, ml, settings }: VisualizationPropTypes<MatrixVisProps>) => {
  const ref = useRef<HTMLDivElement>(null);
  const [graph, setGraph] = useImmer<GraphQueryResult | undefined>(undefined);
  const [highlightNodes, setHighlightNodes] = useState<NodeType[]>([]);
  const [highlightedLinks, setHighlightedLinks] = useState<LinkType[]>([]);

  useEffect(() => {
    if (data) {
      setGraph(data);
    }
  }, [data, ml]);

  return (
    <>
      <div className="h-full w-full overflow-hidden" ref={ref}>
        <MatrixPixi graph={graph} highlightNodes={highlightNodes} highlightedLinks={highlightedLinks} settings={settings} />
      </div>
    </>
  );
});

const MatrixSettings = ({ settings, updateSettings }: VisualizationSettingsPropTypes<MatrixVisProps>) => {
  return (
    <SettingsContainer>
      <Input
        type="dropdown"
        label="Configure marks"
        value={settings.marks}
        options={['rect', 'circle']}
        onChange={(val) => updateSettings({ marks: val as string })}
      />

      <Input
        type="dropdown"
        label="Color"
        value={settings.color}
        options={['blue', 'green']}
        onChange={(val) => updateSettings({ color: val as string })}
      />
    </SettingsContainer>
  );
};

export const MatrixVisComponent: VISComponentType<MatrixVisProps> = {
  displayName: 'MatrixVis',
  description: 'Overview & Details',
  component: MatrixVis,
  settingsComponent: MatrixSettings,
  settings: settings,
};

export default MatrixVisComponent;
